@@ -1,6 +1,4 @@
-
-
-#from __future__ import print_function
+# from __future__ import print_function
 
 # This module is supposed to be imported first
 # it contains global variables used for configuration
@@ -9,7 +7,7 @@
 # try to register printing methods if IPython is running
 
 save_plots = False
-real_type = 'double'
+real_type = "double"
 debug = False
 
 import warnings
@@ -18,19 +16,21 @@ from dolo.misc.termcolor import colored
 
 
 def warning_on_one_line(message, category, filename, lineno, file=None, line=None):
-    return '{}:{}:{}\n    {}\n'.format(
-            colored( category.__name__, 'yellow'),
-            filename,
-            lineno,
-            message)
+    return "{}:{}:{}\n    {}\n".format(
+        colored(category.__name__, "yellow"), filename, lineno, message
+    )
+
+
 warnings.formatwarning = warning_on_one_line
 import numpy
-numpy.seterr(all='ignore')
+
+numpy.seterr(all="ignore")
 
 
 # create temporary directory for compiled functions
 import tempfile, sys
-temp_dir = tempfile.mkdtemp(prefix='dolo_')
+
+temp_dir = tempfile.mkdtemp(prefix="dolo_")
 sys.path.append(temp_dir)
 
 
